/*
 * MPU6050 I2C accelerometer and temperature sensor driver
 * http://cylonjs.com
 *
 * Copyright (c) 2013-14 The Hybrid Group
 * Licensed under the Apache 2.0 license.
*/

// jshint unused:false

"use strict";

var Cylon = require("cylon");

var I2CDriver = require("./i2c-driver");

var MPU6050_RA_ACCEL_XOUT_H = 0x3B,
    MPU6050_RA_PWR_MGMT_1 = 0x6B,
    MPU6050_PWR1_CLKSEL_BIT = 2,
    MPU6050_PWR1_CLKSEL_LENGTH = 3,
    MPU6050_CLOCK_PLL_XGYRO = 0x01,

    MPU6050_GYRO_FS_250 = 0x00,
    MPU6050_RA_GYRO_CONFIG = 0x1B,
    MPU6050_GCONFIG_FS_SEL_LENGTH = 2,
    MPU6050_GCONFIG_FS_SEL_BIT = 4,

    MPU6050_RA_ACCEL_CONFIG = 0x1C,
    MPU6050_ACONFIG_AFS_SEL_BIT = 4,
    MPU6050_ACONFIG_AFS_SEL_LENGTH = 2,
    MPU6050_ACCEL_FS_2 = 0x00,

    MPU6050_RA_PWR_MGMT_1 = 0x6B,
    MPU6050_PWR1_SLEEP_BIT = 6;

/**
 * A MPU6050 Driver
 *
 * @constructor mpu6050
 */
var Mpu6050 = module.exports = function Mpu6050() {
  Mpu6050.__super__.constructor.apply(this, arguments);
  this.address = this.address || 0x68; // DataSheet
  this.commands = {
    get_angular_velocity: this.getAngularVelocity,
    get_acceleration: this.getAcceleration,
    get_motion_and_temp: this.getMotionAndTemp
  };
};

Cylon.Utils.subclass(Mpu6050, I2CDriver);

/**
 * Starts the driver
 *
 * @param {Function} callback triggered when the driver is started
 * @return {null}
 */
Mpu6050.prototype.start = function(callback) {
  // setClockSource
  this._writeBits(MPU6050_RA_PWR_MGMT_1,
                  [MPU6050_PWR1_CLKSEL_BIT,
                   MPU6050_PWR1_CLKSEL_LENGTH,
                   MPU6050_CLOCK_PLL_XGYRO]);

  // setFullScaleGyroRange
  this._writeBits(MPU6050_RA_GYRO_CONFIG,
                  [MPU6050_GCONFIG_FS_SEL_BIT,
                   MPU6050_GCONFIG_FS_SEL_LENGTH,
                   MPU6050_GYRO_FS_250]);

  // setFullScaleAccelRange
  this._writeBits(MPU6050_RA_ACCEL_CONFIG,
                  [MPU6050_ACONFIG_AFS_SEL_BIT,
                   MPU6050_ACONFIG_AFS_SEL_LENGTH,
                   MPU6050_ACCEL_FS_2]);

  // setSleepEnabled
  this._writeBits(MPU6050_RA_PWR_MGMT_1, [MPU6050_PWR1_SLEEP_BIT, false]);

  callback();
  this.emit("start");
};

/**
 * Gets the value of the Angular Velocity
 *
 * @param {Function} callback
 * @return {null}
 * @publish
 */
Mpu6050.prototype.getAngularVelocity = function(callback) {
  this.getMotionAndTemp(callback);
};

/**
 * Gets the value of the Acceleration
 *
 * @param {Function} callback
 * @return {null}
 * @publish
 */
Mpu6050.prototype.getAcceleration = function(callback) {
  this.getMotionAndTemp(callback);
};

/**
 * Gets the value of the Motion.
 *
 * @param {Function} callback
 * @return {null}
 * @publish
 */
Mpu6050.prototype.getMotionAndTemp = function(callback) {
  this.connection.i2cRead(
    this.address,
    MPU6050_RA_ACCEL_XOUT_H,
    14,
    function(err, d) {
      var data = new Buffer(d);
      var ax = data.readInt16BE(0),
          ay = data.readInt16BE(2),
          az = data.readInt16BE(4);

      var temp = data.readInt16BE(6);

      var gx = data.readInt16BE(8),
          gy = data.readInt16BE(10),
          gz = data.readInt16BE(12);

      var values = {
        "a": [ax,ay,az],
        "g": [gx,gy,gz],
        "t": this.convertToCelsius(temp)
      };

      callback(err, values);
    }.bind(this));
};

// The temperature sensor is -40 to +85 degrees Celsius.
// It is a signed integer.
// According to the datasheet:
//   340 per degrees Celsius, -512 at 35 degrees.
// At 0 degrees: -512 - (340 * 35) = -12412
Mpu6050.prototype.convertToCelsius = function (temp) {
  return (temp + 12412.0) / 340.0;
};

Mpu6050.prototype._bitMask = function(bit, bitLength) {
  return ((1 << bitLength) - 1) << (1 + bit - bitLength);
};

Mpu6050.prototype._writeBits = function(func, bit, bitLength, value, callback) {
  var that = this;
  this.connection.i2cRead(this.address, func, 1, function(err, buffer) {
    var mask = that._bitMask(bit, bitLength);
    var newValue = buffer ^ ((buffer ^ (value << bit)) & mask);
    console.log(newValue);
    that.connection.i2cWrite(that.address, func, [newValue], callback);
  });
};
